#include <gtk/gtk.h>
#include <math.h>

#include "gwt_marshallers.h"
#include "gwt_plot_adjustment.h"

#define GWT_PLOT_ADJUSTMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GWT_TYPE_PLOT_ADJUSTMENT, GwtPlotAdjustmentPrivate))


G_DEFINE_TYPE (GwtPlotAdjustment, gwt_plot_adjustment, GTK_TYPE_OBJECT)


typedef struct _GwtPlotAdjustmentPrivate GwtPlotAdjustmentPrivate;

struct _GwtPlotAdjustmentPrivate
{
	gdouble min_value;
	gdouble max_value;
	gdouble start_value;
	gdouble end_value;


	gdouble dev_start_value;
	gdouble dev_end_value;
	gdouble dev_scale;
};

enum
{
	PROP_0,
	PROP_MIN_VALUE,
	PROP_MAX_VALUE,
	PROP_START_VALUE,
	PROP_END_VALUE,
};

enum
{
	CHANGED,
	LAST_SIGNAL
};

static void gwt_plot_adjustment_get_property (GObject      *object,
                                         guint         prop_id,
                                         GValue       *value,
                                         GParamSpec   *pspec);
static void gwt_plot_adjustment_set_property (GObject      *object,
                                         guint         prop_id,
                                         const GValue *value,
                                         GParamSpec   *pspec);

static guint gwt_plot_adjustment_signals[LAST_SIGNAL] = { 0 };

static void
gwt_plot_adjustment_class_init (GwtPlotAdjustmentClass *class)
{
	GObjectClass *obj_class;

	obj_class = G_OBJECT_CLASS (class);

	obj_class->set_property = gwt_plot_adjustment_set_property;
	obj_class->get_property = gwt_plot_adjustment_get_property;

	class->changed = NULL;

	g_object_class_install_property (obj_class,
			PROP_START_VALUE,
			g_param_spec_double ("start_value",
				"Start Value",
				"The start value",
				-G_MAXDOUBLE, 
				G_MAXDOUBLE, 
				0.0, 
				G_PARAM_READWRITE));

	g_object_class_install_property (obj_class,
			PROP_END_VALUE,
			g_param_spec_double ("end_value",
				"End Value",
				"The end value",
				-G_MAXDOUBLE, 
				G_MAXDOUBLE, 
				0.0, 
				G_PARAM_READWRITE));

	g_object_class_install_property (obj_class,
			PROP_END_VALUE,
			g_param_spec_double ("min_value",
				"Min Value",
				"The min value",
				-G_MAXDOUBLE, 
				G_MAXDOUBLE, 
				0.0, 
				G_PARAM_READWRITE));

	g_object_class_install_property (obj_class,
			PROP_END_VALUE,
			g_param_spec_double ("max_value",
				"Max Value",
				"The max value",
				-G_MAXDOUBLE, 
				G_MAXDOUBLE, 
				0.0, 
				G_PARAM_READWRITE));

	/* GwtPlotAdjustment signals */
	gwt_plot_adjustment_signals[CHANGED] =
		g_signal_new (
			"changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotAdjustmentClass, changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);

	g_type_class_add_private (obj_class, sizeof (GwtPlotAdjustmentPrivate));
}

static void
gwt_plot_adjustment_init (GwtPlotAdjustment *adj)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_if_fail (adj != NULL);
	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	priv->min_value = -G_MAXDOUBLE;
	priv->max_value = G_MAXDOUBLE;
	priv->start_value = -1.0;
	priv->end_value = 1.0;
}



gdouble 
gwt_plot_adjustment_get_min_value (GwtPlotAdjustment *adj)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_val_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj), 0.0);

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	return priv->min_value;
}

gdouble 
gwt_plot_adjustment_get_max_value (GwtPlotAdjustment *adj)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_val_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj), 0.0);

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	return priv->max_value;
}

gdouble 
gwt_plot_adjustment_get_start_value (GwtPlotAdjustment *adj)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_val_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj), 0.0);

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	return priv->start_value;
}

gdouble 
gwt_plot_adjustment_get_end_value (GwtPlotAdjustment *adj)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_val_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj), 0.0);

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	return priv->end_value;
}

void
gwt_plot_adjustment_set_min_value (GwtPlotAdjustment *adj, gdouble value)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	if (value != priv->min_value) {
		priv->min_value = value;

		if (priv->start_value < value)
			priv->start_value = value;

		g_signal_emit (adj, gwt_plot_adjustment_signals[CHANGED], 0);
		g_object_notify (G_OBJECT (adj), "min_value");
	}
}

void
gwt_plot_adjustment_set_max_value (GwtPlotAdjustment *adj, gdouble value)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	if (value != priv->max_value) {
		priv->max_value = value;

		if (priv->end_value > value)
			priv->end_value = value;
		
		g_signal_emit (adj, gwt_plot_adjustment_signals[CHANGED], 0);
		g_object_notify (G_OBJECT (adj), "max_value");
	}
}

void
gwt_plot_adjustment_set_start_value (GwtPlotAdjustment *adj, gdouble value)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	if (value < priv->min_value)
		value = priv->min_value;

	if (value != priv->start_value) {
		priv->start_value = value;

		g_signal_emit (adj, gwt_plot_adjustment_signals[CHANGED], 0);
		g_object_notify (G_OBJECT (adj), "start_value");
	}
}

void
gwt_plot_adjustment_set_end_value (GwtPlotAdjustment *adj, gdouble value)
{
	GwtPlotAdjustmentPrivate *priv;

	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	if (value > priv->max_value)
		value = priv->max_value;

	if (value != priv->end_value) {
		priv->end_value = value;

		g_signal_emit (adj, gwt_plot_adjustment_signals[CHANGED], 0);
		g_object_notify (G_OBJECT (adj), "end_value");
	}
}

static void
gwt_plot_adjustment_get_property (GObject *object, guint prop_id, GValue *value, 
	                     GParamSpec *pspec)
{
 	GwtPlotAdjustmentPrivate *priv;
	GwtPlotAdjustment *adjustment = GWT_PLOT_ADJUSTMENT (object);

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adjustment);

	switch (prop_id)
	{
	case PROP_MIN_VALUE:
		g_value_set_double (value, priv->min_value);
		break;
	case PROP_MAX_VALUE:
		g_value_set_double (value, priv->max_value);
		break;
	case PROP_START_VALUE:
		g_value_set_double (value, priv->start_value);
		break;
	case PROP_END_VALUE:
		g_value_set_double (value, priv->end_value);
		break;

	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
gwt_plot_adjustment_set_property (GObject * object, guint prop_id, 
	                     const GValue * value, GParamSpec * pspec)
{
	GwtPlotAdjustment *adjustment = GWT_PLOT_ADJUSTMENT (object);
	gdouble double_value = g_value_get_double (value);

	switch (prop_id) 
	{	
	case PROP_MIN_VALUE:
		gwt_plot_adjustment_set_min_value (adjustment, double_value);
		break;
	case PROP_MAX_VALUE:
		gwt_plot_adjustment_set_max_value (adjustment, double_value);
		break;
	case PROP_START_VALUE:
		gwt_plot_adjustment_set_start_value (adjustment, double_value);
		break;
	case PROP_END_VALUE:
		gwt_plot_adjustment_set_end_value (adjustment, double_value);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

void gwt_plot_adjustment_zoom(GwtPlotAdjustment *adj, gdouble start_value, gdouble end_value)
{
	GwtPlotAdjustmentPrivate *priv;
	gboolean end_changed = FALSE, start_changed = FALSE;
	gdouble diff;

	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);


	if (start_value > end_value) {
		gdouble tmp = start_value;
		start_value = end_value;
		end_value = tmp;
	}
	
	if (start_value < priv->min_value)
		start_value = priv->min_value;

	if (end_value > priv->max_value)
		end_value = priv->max_value;

	diff = end_value - start_value;
	if ((end_value - start_value) < 0.000001) 
		end_value = start_value + 0.000001;

	if (start_value != priv->start_value) {
		priv->start_value = start_value;
		start_changed = TRUE;
	}

	if (end_value != priv->end_value) {
		priv->end_value = end_value;
		end_changed = TRUE;
		g_object_notify (G_OBJECT (adj), "end_value");
	}

	if (start_changed)
		g_object_notify (G_OBJECT (adj), "start_value");

	if (end_changed)
		g_object_notify (G_OBJECT (adj), "end_value");

	if (end_changed || start_changed)
		g_signal_emit (adj, gwt_plot_adjustment_signals[CHANGED], 0);
}

void gwt_plot_adjustment_move(GwtPlotAdjustment *adj, gdouble delta)
{
	GwtPlotAdjustmentPrivate *priv;
	gdouble d;

	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));

	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	d = priv->end_value - priv->start_value;
	priv->start_value += delta;
	if (priv->start_value < priv->min_value)
		priv->start_value = priv->min_value;		

	priv->end_value = priv->start_value + d;

	if (priv->end_value > priv->max_value) {
		priv->end_value = priv->max_value;
	
		priv->start_value = priv->end_value - d;
		if (priv->start_value < priv->min_value)
			priv->start_value = priv->min_value;
	}

	g_signal_emit (adj, gwt_plot_adjustment_signals[CHANGED], 0);
	g_object_notify (G_OBJECT (adj), "start_value");
	g_object_notify (G_OBJECT (adj), "end_value");
}

GtkObject *
gwt_plot_adjustment_new (gdouble start_value, gdouble end_value)
{
	return g_object_new (GWT_TYPE_PLOT_ADJUSTMENT, 
			"start_value", start_value,
			"end_value", end_value,
			NULL);
}

void gwt_plot_adjustment_set_device_interval(GwtPlotAdjustment *adj, gint start, gint end)
{
	GwtPlotAdjustmentPrivate *priv;
	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);

	priv->dev_start_value = start;
	priv->dev_end_value = end;
	priv->dev_scale = (priv->dev_end_value - priv->dev_start_value) / (priv->end_value - priv->start_value);
}

gdouble gwt_plot_adjustment_device_to_user(GwtPlotAdjustment *adj, gint v)
{
	GwtPlotAdjustmentPrivate *priv;
	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);
	
	return (((gdouble)v - priv->dev_start_value) / priv->dev_scale) + priv->start_value;
}

gint gwt_plot_adjustment_user_to_device(GwtPlotAdjustment *adj, gdouble v)
{
	GwtPlotAdjustmentPrivate *priv;
	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);
	
	return round(((v - priv->start_value) * priv->dev_scale) + priv->dev_start_value);
}

gdouble gwt_plot_adjustment_device_to_user_distance(GwtPlotAdjustment *adj, gint v)
{
	GwtPlotAdjustmentPrivate *priv;
	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);
	
	return ((gdouble)v / priv->dev_scale);
}

gint gwt_plot_adjustment_user_to_device_distance(GwtPlotAdjustment *adj, gdouble v)
{
	GwtPlotAdjustmentPrivate *priv;
	priv = GWT_PLOT_ADJUSTMENT_GET_PRIVATE (adj);
	
	return round(v * priv->dev_scale);
}

