#include <gtk/gtk.h>
#include <math.h>
#include <time.h>

#include "gwt_marshallers.h"
#include "gwt_plot_cursor.h"
#include "gwt_plot_item.h"
#include "gwt_plot_data.h"

#define GWT_PLOT_CURSOR_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GWT_TYPE_PLOT_CURSOR, GwtPlotCursorPrivate))


G_DEFINE_TYPE (GwtPlotCursor, gwt_plot_cursor, GWT_TYPE_PLOT_ITEM)

static void gwt_plot_cursor_draw_impl (GwtPlotItem* item, cairo_t *cr, int w, int h);

typedef struct _GwtPlotCursorPrivate GwtPlotCursorPrivate;

struct _GwtPlotCursorPrivate
{	
	GwtPlotData* data;
};

enum
{
	CHANGED,
	VALUE_CHANGED,
	LAST_SIGNAL
};

//static guint gwt_plot_grid_signals[LAST_SIGNAL] = { 0 };


static void
gwt_plot_cursor_class_init (GwtPlotCursorClass *class)
{
	GObjectClass *obj_class;
	GwtPlotItemClass *item_class;
	//GtkWidgetClass *widget_class;

	obj_class = G_OBJECT_CLASS (class);
	item_class = GWT_PLOT_ITEM_CLASS(class);

	/* virtual functions */
	item_class->draw = gwt_plot_cursor_draw_impl;

#if 0
	/* GwtPlotItem signals */
	gwt_plot_item_signals[CHANGED] =
		g_signal_new (
			"changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotItemClass, changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);

	gwt_plot_item_signals[VALUE_CHANGED] =
		g_signal_new (
			"value_changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotItemClass, value_changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);
#endif

	g_type_class_add_private (obj_class, sizeof (GwtPlotCursorPrivate));
}

static void
gwt_plot_cursor_init (GwtPlotCursor *cursor)
{
}

static void
gwt_plot_cursor_draw_impl (GwtPlotItem* item, cairo_t *cr, int w, int h)
{
	GwtPlotCursorPrivate *priv;
	GwtPlotAdjustment* adj;
	gdouble x_min, x_max, y_min, y_max;
	gdouble x_scale, y_scale;

	g_return_if_fail (item != NULL);
	g_return_if_fail (GWT_IS_PLOT_CURSOR (item));
	g_return_if_fail (cr != NULL);

  	priv = GWT_PLOT_CURSOR_GET_PRIVATE (item);

	adj = gwt_plot_item_get_x_adjustment (item);
	g_return_if_fail (adj != NULL);
	x_min = gwt_plot_adjustment_get_start_value(adj);
	x_max = gwt_plot_adjustment_get_end_value(adj);

	adj = gwt_plot_item_get_y_adjustment (item);
	g_return_if_fail (adj != NULL);
	y_min = gwt_plot_adjustment_get_start_value(adj);
	y_max = gwt_plot_adjustment_get_end_value(adj);

	x_scale = ((gdouble)w) / (x_max - x_min);
	y_scale = ((gdouble)h) / (y_max - y_min);
	
	cairo_set_source_rgb (cr, 0, 0, 0);
	cairo_set_line_width (cr, 1.0);

	cairo_save(cr);

	cairo_set_antialias (cr , CAIRO_ANTIALIAS_NONE);
	
	cairo_scale(cr, x_scale, y_scale);
	cairo_translate(cr, -x_min, -y_min);

	cairo_move_to (cr, 0.0, y_min);
	cairo_line_to(cr, 0.0, y_max);

	cairo_move_to (cr, x_min, 0.0);
	cairo_line_to(cr, x_max, 0.0);

	cairo_restore(cr);	

	cairo_stroke (cr);
}

GtkObject *
gwt_plot_cursor_new (void)
{
	return g_object_new (GWT_TYPE_PLOT_CURSOR, NULL);
}

