#include <gtk/gtk.h>

#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "gwt_marshallers.h"
#include "gwt_plot_data.h"

#define GWT_PLOT_DATA_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GWT_TYPE_PLOT_DATA, GwtPlotDataPrivate))


G_DEFINE_TYPE (GwtPlotData, gwt_plot_data, GTK_TYPE_OBJECT)


typedef struct _GwtPlotDataPrivate GwtPlotDataPrivate;

struct _GwtPlotDataPrivate
{
	GwtPlotDataPoint* data;
	guint data_len;	/* the number of entries used in the data array */
	guint data_size;	/* the size of the data array */
	gboolean data_is_sorted; /* TRUE is the data array is sorted in X order */
};

enum
{
	CHANGED,
	VALUE_CHANGED,
	LAST_SIGNAL
};

static guint gwt_plot_data_signals[LAST_SIGNAL] = { 0 };

static void
gwt_plot_data_class_init (GwtPlotDataClass *class)
{
	GObjectClass *obj_class;

	obj_class = G_OBJECT_CLASS (class);


	/* GwtPlotData signals */
	gwt_plot_data_signals[CHANGED] =
		g_signal_new (
			"changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotDataClass, changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);

	gwt_plot_data_signals[VALUE_CHANGED] =
		g_signal_new (
			"value_changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotDataClass, value_changed),
			NULL, NULL,
			_gwt_marshal_VOID__DOUBLE_DOUBLE,
			G_TYPE_NONE, 2,
			G_TYPE_DOUBLE,
			G_TYPE_DOUBLE);

	g_type_class_add_private (obj_class, sizeof (GwtPlotDataPrivate));
}


static void
gwt_plot_data_resize(GwtPlotData *data, guint size)
{
	GwtPlotDataPrivate *priv;
	GwtPlotDataPoint* old;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	old = priv->data;

	priv->data_size = size;
	priv->data = g_new (GwtPlotDataPoint, priv->data_size);
	
	if (old != NULL) {
		memcpy (priv->data, old, priv->data_len * sizeof(GwtPlotDataPoint));
		g_free(old);
	}
}

static gint
gwt_plot_data_point_cmp(gconstpointer p1, gconstpointer p2)
{
	GwtPlotDataPoint* a = (GwtPlotDataPoint*)p1;
	GwtPlotDataPoint* b = (GwtPlotDataPoint*)p2;

	if (a->x < b->x)
		return -1;
	else if (a->x > b->x)
		return 1;
	else
		return 0;
}

static void
gwt_plot_data_sort (GwtPlotData *data)
{
	GwtPlotDataPrivate *priv;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	qsort(priv->data, priv->data_len, sizeof(GwtPlotDataPoint), gwt_plot_data_point_cmp);	
}

static void
gwt_plot_data_init (GwtPlotData *data)
{
	GwtPlotDataPrivate *priv;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	priv->data_len = 0;
	priv->data_size = 0;
	priv->data =  NULL;
}

GtkObject *
gwt_plot_data_new (void)
{
	return g_object_new (GWT_TYPE_PLOT_DATA, NULL);
}

void 
gwt_plot_data_insert( GwtPlotData* data, GwtPlotDataPoint* data_point)
{
	GwtPlotDataPrivate *priv;
	gint i;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	if (priv->data_len == priv->data_size) {
		gwt_plot_data_resize(data, priv->data_size + 1024);
	}

	if ((priv->data_len > 0 && priv->data[priv->data_len - 1].x < data_point->x) || (priv->data_len == 0)) {
		/* we just want to add a new vlaue to the end */
		priv->data[priv->data_len++] = *data_point;
		//g_debug("gwt_plot_data_insert %f at end", data_point->x);
	} else {
		/* we have to check if there is already and entry for value x */
		i = gwt_plot_data_find(data, data_point->x, NULL);
		if (i == -1) {
			/* no entry found */
			priv->data[priv->data_len++] = *data_point;
			gwt_plot_data_sort(data);
			//g_debug("gwt_plot_data_insert %f in the middle", data_point->x);
		} else {
			/* update the found entry */
			priv->data[i] = *data_point;
			//g_debug("gwt_plot_data_insert %f at %d", data_point->x, i);
		}
	}


	//if (priv->data_len > 200000)
	//	gwt_plot_data_remove(data, 0);

	/* emit change signal */
	g_signal_emit (data, gwt_plot_data_signals[VALUE_CHANGED], 0, data_point->x, data_point->y );
}

void 
gwt_plot_data_remove( GwtPlotData* data, guint index)
{
	GwtPlotDataPrivate *priv;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	if ((priv->data_len < 1) || (index >= priv->data_len))
		return;

	memmove (&priv->data[index], &priv->data[index+1], (priv->data_len - index) * sizeof(GwtPlotDataPoint));
	
	priv->data_len--;	
}

void 
gwt_plot_data_remove_all (GwtPlotData* data)
{

}

void 
gwt_plot_data_remove_range (GwtPlotData* data, guint index, guint count)
{

}

gint 
gwt_plot_data_get (GwtPlotData* data, guint index, GwtPlotDataPoint* data_point)
{
	GwtPlotDataPrivate *priv;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	if (index >= priv->data_len)
		return -1;

	if (data_point != NULL) {
		*data_point = priv->data[index];
	}

	return index;
}


/*
 * Perform a binary search. From NetBSD 
 *
 * The code below is a bit sneaky.  After a comparison fails, we
 * divide the work in half by moving either left or right. If lim
 * is odd, moving left simply involves halving lim: e.g., when lim
 * is 5 we look at item 2, so we change lim to 2 so that we will
 * look at items 0 & 1.  If lim is even, the same applies.  If lim
 * is odd, moving right again involes halving lim, this time moving
 * the base up one item past p: e.g., when lim is 5 we change base
 * to item 3 and make lim 2 so that we will look at items 3 and 4.
 * If lim is even, however, we have to shrink it by one before
 * halving: e.g., when lim is 4, we still looked at item 2, so we
 * have to make lim 3, then halve, obtaining 1, so that we will only
 * look at item 3.
 */
static 
void * gwt_plot_data_bsearch(const void* key, const void* base0, size_t nmemb, size_t size, int (*compar)(const void *, const void *))
{
	const char *base = base0;
	size_t lim;
	int cmp;
	const void *p = NULL;

	for (lim = nmemb; lim != 0; lim >>= 1) {
		p = base + (lim >> 1) * size;
		cmp = (*compar)(key, p);
		if (cmp == 0) {
			return (void*)p;
		}
		if (cmp > 0) {	/* key > p: move right */
			base = (const char *)p + size;
			lim--;
		}		/* else move left */
	}
	
	return (void*)base;
}

gint 
gwt_plot_data_find ( GwtPlotData* data, gdouble x, GwtPlotDataPoint* data_point)
{
	GwtPlotDataPoint key;
	GwtPlotDataPoint* res;
	GwtPlotDataPrivate *priv;
	gint i;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	key.x = x;
	key.y = 0.0;

	res = gwt_plot_data_bsearch(&key, priv->data, priv->data_len, sizeof(GwtPlotDataPoint), gwt_plot_data_point_cmp);
	if (res == NULL) {
		return -1;
	} else {
		i = ((gulong)res - (gulong)priv->data) / sizeof(GwtPlotDataPoint);

		if (priv->data[i].x == x) {
			//g_debug("gwt_plot_data_find x=%f i=%d found x=%f", x, i, priv->data[i].x);

			if (data_point != NULL)
				*data_point = priv->data[i];
			
			return i;
		} else {
			return -1;
		}
	}
}

gint gwt_plot_data_find_smaller ( GwtPlotData* data, gdouble x, GwtPlotDataPoint* data_point)
{
	GwtPlotDataPoint key;
	GwtPlotDataPoint* res;
	GwtPlotDataPrivate *priv;
	gint i = -1;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	key.x = x;
	key.y = 0.0;

	res = gwt_plot_data_bsearch(&key, priv->data, priv->data_len, sizeof(GwtPlotDataPoint), gwt_plot_data_point_cmp);
	if (res != NULL) {
		i = res - priv->data;

		if (priv->data[i].x > x)
			i--;
	}

	if ((data_point != NULL) && (i >= 0) && (i < priv->data_len))
		*data_point = priv->data[i];			

	return i;
}

gint gwt_plot_data_find_larger ( GwtPlotData* data, gdouble x, GwtPlotDataPoint* data_point)
{
	return 0;
}

void gwt_plot_data_find_min_max_x( GwtPlotData* data, gdouble* min_x, gdouble* max_x)
{
	GwtPlotDataPrivate *priv;

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	if (min_x != NULL)
		*min_x = priv->data[0].x;

	if (max_x != NULL)
		*max_x = priv->data[priv->data_len - 1].x;
}


void gwt_plot_data_find_min_max_y( GwtPlotData* data, gdouble start_x, gdouble end_x, GwtPlotDataPoint* min_y, GwtPlotDataPoint* max_y)
{
	GwtPlotDataPrivate *priv;
	GwtPlotDataPoint data_point, y_min, y_max;
	int i;

	y_min.y = G_MAXDOUBLE;
	y_min.x = -G_MAXDOUBLE;
	y_max.y = -G_MAXDOUBLE;
	y_max.x = G_MAXDOUBLE;
	

	priv = GWT_PLOT_DATA_GET_PRIVATE (data);

	i = gwt_plot_data_find_smaller (data, start_x , &data_point);
	if ( i < 0)
		i = 0;

	for ( ;i < priv->data_len; i++) {
		if (priv->data[i].x < start_x)
			continue;

		if (priv->data[i].x > end_x)
			break;

		if (priv->data[i].y < y_min.y)
			y_min = priv->data[i];

		if (priv->data[i].y > y_max.y)
			y_max = priv->data[i];
	}

	if (min_y != NULL)
		*min_y = y_min;

	if (max_y != NULL)
		*max_y = y_max;
}




