#include <gtk/gtk.h>
#include <math.h>
#include <time.h>

#include "gwt_marshallers.h"
#include "gwt_plot_grid.h"
#include "gwt_plot_item.h"
#include "gwt_plot_data.h"

#define GWT_PLOT_GRID_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GWT_TYPE_PLOT_GRID, GwtPlotGridPrivate))


G_DEFINE_TYPE (GwtPlotGrid, gwt_plot_grid, GWT_TYPE_PLOT_ITEM)

static void gwt_plot_grid_draw_impl (GwtPlotItem* item, cairo_t *cr, int w, int h);

typedef struct _GwtPlotGridPrivate GwtPlotGridPrivate;

struct _GwtPlotGridPrivate
{	
	GwtPlotData* data;
};

enum
{
	CHANGED,
	VALUE_CHANGED,
	LAST_SIGNAL
};

//static guint gwt_plot_grid_signals[LAST_SIGNAL] = { 0 };


static void
gwt_plot_grid_class_init (GwtPlotGridClass *class)
{
	GObjectClass *obj_class;
	GwtPlotItemClass *item_class;
	//GtkWidgetClass *widget_class;

	obj_class = G_OBJECT_CLASS (class);
	item_class = GWT_PLOT_ITEM_CLASS(class);

	/* virtual functions */
	item_class->draw = gwt_plot_grid_draw_impl;

#if 0
	/* GwtPlotItem signals */
	gwt_plot_item_signals[CHANGED] =
		g_signal_new (
			"changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotItemClass, changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);

	gwt_plot_item_signals[VALUE_CHANGED] =
		g_signal_new (
			"value_changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotItemClass, value_changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);
#endif

	g_type_class_add_private (obj_class, sizeof (GwtPlotGridPrivate));
}

static void
gwt_plot_grid_init (GwtPlotGrid *grid)
{
}

static void
gwt_plot_grid_draw_impl (GwtPlotItem* item, cairo_t *cr, int w, int h)
{
	GwtPlotGridPrivate *priv;
	GwtPlotAdjustment *x_adj, *y_adj;
	gdouble x_0, y_0;

	g_return_if_fail (item != NULL);
	g_return_if_fail (GWT_IS_PLOT_GRID (item));
	g_return_if_fail (cr != NULL);

  	priv = GWT_PLOT_GRID_GET_PRIVATE (item);

	if (!GWT_PLOT_ITEM(item)->is_visible)
		return;

	x_adj = gwt_plot_item_get_x_adjustment (item);
	g_return_if_fail (x_adj != NULL);
	gwt_plot_adjustment_set_device_interval(x_adj, 0, w);
	x_0 = gwt_plot_adjustment_user_to_device(x_adj, 0.0);

	y_adj = gwt_plot_item_get_y_adjustment (item);
	g_return_if_fail (y_adj != NULL);
	gwt_plot_adjustment_set_device_interval(y_adj, 0, h);
	y_0 = h - gwt_plot_adjustment_user_to_device(y_adj, 0.0);

	cairo_set_source_rgb (cr, GWT_PLOT_ITEM(item)->color[0], GWT_PLOT_ITEM(item)->color[1], GWT_PLOT_ITEM(item)->color[2]);
	cairo_set_line_width (cr, 1.0);

	cairo_save(cr);

	cairo_set_antialias (cr , CAIRO_ANTIALIAS_NONE);
	
	if (x_0 > w / 2)
		x_0 -= 0.5;
	else
		x_0 += 0.5;

	if (x_0 > 0 && x_0 < w){
		cairo_move_to (cr, x_0, 0);
		cairo_line_to(cr, x_0, h);
	}

	if (y_0 > h / 2)
		y_0 -= 0.5;
	else
		y_0 += 0.5;

	if (y_0 > 0 && y_0 < h) {
		cairo_move_to (cr, 0, y_0);
		cairo_line_to(cr, w, y_0);
	}

	cairo_restore(cr);	

	cairo_stroke (cr);
}

GtkObject *
gwt_plot_grid_new (void)
{
	return g_object_new (GWT_TYPE_PLOT_GRID, NULL);
}

