#include <gtk/gtk.h>
#include <math.h>
#include <time.h>

#include "gwt_marshallers.h"
#include "gwt_plot_item.h"

#define GWT_PLOT_ITEM_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GWT_TYPE_PLOT_ITEM, GwtPlotItemPrivate))


G_DEFINE_ABSTRACT_TYPE (GwtPlotItem, gwt_plot_item, GTK_TYPE_OBJECT)


typedef struct _GwtPlotItemPrivate GwtPlotItemPrivate;

struct _GwtPlotItemPrivate
{
	GwtPlotAdjustment* x_adjustment;
	GwtPlotAdjustment* y_adjustment;
};

enum
{
	CHANGED,
	LAST_SIGNAL
};

static guint gwt_plot_item_signals[LAST_SIGNAL] = { 0 };

static void
gwt_plot_item_class_init (GwtPlotItemClass *class)
{
	GObjectClass *obj_class;

	obj_class = G_OBJECT_CLASS (class);

	/* pure virtual functions */
	class->draw = NULL; 

	/* GwtPlotItem signals */
	gwt_plot_item_signals[CHANGED] =
		g_signal_new (
			"changed",
			G_OBJECT_CLASS_TYPE (class),
			G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE,
			G_STRUCT_OFFSET (GwtPlotItemClass, changed),
			NULL, NULL,
			_gwt_marshal_VOID__VOID,
			G_TYPE_NONE, 0);

	g_type_class_add_private (obj_class, sizeof (GwtPlotItemPrivate));
}

static void
gwt_plot_item_init (GwtPlotItem *item)
{
	item->color[0] = 0.0;
	item->color[1] = 0.0;
	item->color[2] = 0.0;
	item->color[3] = 1.0;

	item->is_visible = TRUE;
}

static void
gwt_plot_item_x_adjustment_changed_cb(GwtPlotAdjustment* adj, GwtPlotItem* item)
{
	g_return_if_fail (adj != NULL);
	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));
	g_return_if_fail (item != NULL);
	g_return_if_fail (GWT_IS_PLOT_ITEM (item));

	gwt_plot_item_emit_changed(item);
}

static void
gwt_plot_item_y_adjustment_changed_cb(GwtPlotAdjustment* adj, GwtPlotItem* item)
{
	g_return_if_fail (adj != NULL);
	g_return_if_fail (GWT_IS_PLOT_ADJUSTMENT (adj));
	g_return_if_fail (item != NULL);
	g_return_if_fail (GWT_IS_PLOT_ITEM (item));

	gwt_plot_item_emit_changed(item);
}

GtkObject *
gwt_plot_item_new (void)
{
	return g_object_new (GWT_TYPE_PLOT_ITEM, NULL);
}

void 
gwt_plot_item_set_x_adjustment (GwtPlotItem* item, GwtPlotAdjustment* adj)
{
	GwtPlotItemPrivate *priv;

	priv = GWT_PLOT_ITEM_GET_PRIVATE (item);

	if (priv->x_adjustment != NULL) {
		g_signal_handlers_disconnect_by_func(priv->x_adjustment, G_CALLBACK (gwt_plot_item_x_adjustment_changed_cb), item);

		g_object_unref (priv->x_adjustment);
	}

	g_object_ref_sink (adj);
	priv->x_adjustment = adj;	

	g_signal_connect (priv->x_adjustment, "changed", G_CALLBACK (gwt_plot_item_x_adjustment_changed_cb), item);
}

GwtPlotAdjustment* 
gwt_plot_item_get_x_adjustment(GwtPlotItem* item)
{
	GwtPlotItemPrivate *priv;

	priv = GWT_PLOT_ITEM_GET_PRIVATE (item);

	return priv->x_adjustment;
}

void 
gwt_plot_item_set_y_adjustment (GwtPlotItem* item, GwtPlotAdjustment* adj)
{
	GwtPlotItemPrivate *priv;

	priv = GWT_PLOT_ITEM_GET_PRIVATE (item);

	if (priv->y_adjustment != NULL) {
		g_signal_handlers_disconnect_by_func(priv->x_adjustment, G_CALLBACK (gwt_plot_item_y_adjustment_changed_cb), item);

		g_object_unref (priv->y_adjustment);
	}

	g_object_ref_sink (adj);
	priv->y_adjustment = adj;	

	g_signal_connect (priv->y_adjustment, "changed", G_CALLBACK (gwt_plot_item_y_adjustment_changed_cb), item);
}

GwtPlotAdjustment* 
gwt_plot_item_get_y_adjustment(GwtPlotItem* item)
{
	GwtPlotItemPrivate *priv;

	priv = GWT_PLOT_ITEM_GET_PRIVATE (item);

	return priv->y_adjustment;
}

void gwt_plot_item_draw( GwtPlotItem* item, cairo_t *cr, int w, int h)
{
	GWT_PLOT_ITEM_GET_CLASS (item)->draw (item, cr, w, h);
}

void 
gwt_plot_item_emit_changed(GwtPlotItem* item)
{
	g_signal_emit (item, gwt_plot_item_signals[CHANGED], 0);
}

void gwt_plot_item_set_rgba (GwtPlotItem *item, gdouble r, gdouble g, gdouble b, gdouble a)
{
	g_return_if_fail (item != NULL);
	g_return_if_fail (GWT_IS_PLOT_ITEM (item));

	item->color[0] = r;
	item->color[1] = g;
	item->color[2]= b;
	item->color[3] = a;

	gwt_plot_item_emit_changed(GWT_PLOT_ITEM(item));
}

void gwt_plot_item_set_color (GwtPlotItem *item, GdkColor* color)
{
	g_return_if_fail (item != NULL);
	g_return_if_fail (GWT_IS_PLOT_ITEM (item));
	g_return_if_fail (color != NULL);
	
	item->color[0] = (double)color->red / 65535.0;
	item->color[1] = (double)color->green / 65535.0;
	item->color[2]= (double)color->blue / 65535.0;
	item->color[3] = 1.0;

	gwt_plot_item_emit_changed(GWT_PLOT_ITEM(item));
}

void gwt_plot_item_show(GwtPlotItem *item) {
	GwtPlotItemPrivate *priv;

	priv = GWT_PLOT_ITEM_GET_PRIVATE (item);

	item->is_visible = TRUE;	

	gwt_plot_item_emit_changed(GWT_PLOT_ITEM(item));
}

void gwt_plot_item_hide(GwtPlotItem *item) {
	GwtPlotItemPrivate *priv;

	priv = GWT_PLOT_ITEM_GET_PRIVATE (item);

	item->is_visible = FALSE;	

	gwt_plot_item_emit_changed(GWT_PLOT_ITEM(item));
}

